# Ranking Module
 
To use you'll need to instanciate a Rater, and call its method sortByRate with an Item array. (see below)

The ranking module will generate ranking score for each item, order them by that score and return this ordered list.

- **Input:** Array of Item
- **Output:** Ordered array of Item, most ranked coming first

### Ranking::Item Class

- +name+: name of object
- +views+: view count
- +downloads+: download count
- +likes+: like count
- +info+: anything, just for convinience of sorting it together, won't be modified


### Usage Example

```ruby

items = []
items << Ranking::Item.new("ax",1,1,1,["otherStuffA","lala",'a'])
items << Ranking::Item.new("bx",2,2,2,[1,2,3,"teste","lblb",'b'])
items << Ranking::Item.new("cx",1,1,99,[3,2,1,"teste","lclc",'c'])

rater = Ranking::Rater.new(Ranking::Strategies::BasicRater.new(1000, 1, 100))
rater.sortByRate(items).each do |i|
    p i.name
end
```

## Output
 - ordered array of Items, starting from the most ranked to the least

``` ruby
rater = Ranking::Rater.new(
      Ranking::Strategies::BasicRater.new( 
         #define sorting weights
         positionWeight = 1000,
         useWeight = 1,
         likeWeight = 100
      )
    )

sorted_item_array = rater.sortByRate(items) # Returns sorted array of items
```

### Sorting Weights

| **Weight**    | **Ranking Points**  |
| :----------   | :--------------  |
| ***Position*** | If two items A and B are in positions 1(first) and 2, respectively, A will have [positionWeight] more rank points than B. This basically means that the higher the position weight the more dificult is to items change positions by likes, views and downloads. |
| ***Use***      | Ranking Points = [useWeight] * minimum ( views , downloads ) |
| ***Like***     | For each like, items receive [likeWeight] ranking points     |
