namespace :orientdb do
  desc "Orient DB integration tasks"

  task migrate: :environment do
    desc "Migrate orient db schema"

    migrations = OrientDb::Migrations.new(OrientDb::Client.instance)
    migrations.run
  end

  task create_learning_object_relations: :environment do
    desc "Create LearningObject relations based on its metadata"

    include RepositoriesProxy

    # Quantity of LearningObjects fetched on each iteration
    limit = 1000
    # Starting point from where LearningObjects will be fetched
    offset = 0

    loop do
      puts " --> importing LearningObjects from #{offset} to #{offset+limit}"

      begin
        # Get LearningObjects from OrientDB (from offset to offset+limit)
        learning_objects = learning_object_repository.all_from_offset_to_limit(offset,limit)
      rescue
        # Sleeps for a while to wait database's recovery
        sleep(30.seconds)
        # Goes to next iteration to retry
        next
      else
        # Terminate loop if there are no more LearningObjects
        break if learning_objects.empty?

        learning_objects.each do |lo|
          metadata = get_unique_matadata_keys(lo)
          subjects = metadata["dc.subject.category"]
          subjects ||= []
          metadata.delete("dc.subject.category")

          subjects_array=[]
          attributes_array=[]

          subjects.each do |subject_name|
            subjects_array << {:name => subject_name}
          end

          metadata.each do |key, value|
            attributes_array << {:key => key, :value => value}
          end

          CreateRelationsWorker.perform_async(lo.id, subjects_array, attributes_array)
        end

        offset += limit
      end
    end

  end

  def get_unique_matadata_keys(lo)
    hash = {}
    lo.metadata.each do |m|
      unless hash.has_key?(m['key'])
        hash[m['key']] = lo.get_metadata_values_of(m['key'])
      end
    end
    hash
  end

end
