module Thumbnail
  module Creation

    include Formats

    def generate_thumbnail(input, filename, size)
      unless accepted_formats.include? file_format(filename)
        return default_thumbnail
      else
        thumbnail = thumbnail_path(filename, size)
        output = "#{root_dir}#{thumbnail}"
        begin
          if accepted_video_formats.include? File.extname(input)
            generate_video_thumbnail(input, output, size)
          else
            generate_image_thumbnail(input, output, size)
          end
        rescue
          return default_thumbnail
        else
          return thumbnail
        end
      end

    end

    def default_thumbnail
      @default_thumbnail ||= nil
    end

    private

    def generate_video_thumbnail(input,output,size)
      movie = FFMPEG::Movie.new(input)
      frame = (movie.duration * 25/100).floor
      movie.screenshot(output,
                        { seek_time: frame, resolution: size },
                        preserve_aspect_ratio: :width)
    end

    def generate_image_thumbnail(input,output,size)
      # Read the image and resize it. The `change_geometry' method
      # computes the new image geometry and yields to a block. The
      # return value of the block is the return value of the method.
      img = Magick::Image.read(input)[0]
      img.change_geometry!(size) { |cols, rows| img.thumbnail! cols, rows }
      img.write(output)
    end

    def encode_hash_from(object)
      Digest::SHA1.hexdigest object
    end

    def thumbnail_path(filename, size)
      thumbnail_name = encode_hash_from filename
      thumbnail_path = "#{thumbnails_dir}/#{thumbnail_name}_#{size}.jpg"
    end

    def thumbnails_dir
      @thumbnails_dir ||= "/thumbnails"
    end

    def root_dir
      @root_dir ||= Rails.root.join('public')
    end

  end
end
